package Server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import Client.IClientInterface;
import java.util.LinkedList;

public class ServerImpl extends UnicastRemoteObject implements IServerInterface{

    private final LinkedList<IClientInterface> clients;

    public ServerImpl() throws RemoteException {
        clients = new LinkedList<IClientInterface>();
    }

    public void register(IClientInterface client) throws RemoteException{
        synchronized(clients){
            System.out.println("Client " + client.hashCode() + " registered");
            clients.add(client);
        }
    }

    public void sendMessage(IClientInterface inClient, String str) throws RemoteException{
        synchronized(clients){
            System.out.println("Client " + inClient.hashCode() + " send message to");
            for(IClientInterface client : clients){                
                if (!client.equals(inClient)){
                    try{
                     client.getMessage(str);
                     System.out.println("\tclient " + client.hashCode());
                    }
                    catch(RemoteException ex){
                       // Proper error handling
                    }
                }
            }
        }
    }

    public static void main(String[] args) {
        try {            
            Registry rmi = LocateRegistry.createRegistry(PORT);
            IServerInterface server = new ServerImpl();
            rmi.bind(LOOKUP_NAME, server);
        } catch (Exception ex) {
            // Proper error handling
        }
    }

}
