package Server;

import Shared.CheckSum;
import Shared.Zipper;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;



import Shared.IDownloadSession;
import Shared.ISimpleServer;


public class DownloadManager extends UnicastRemoteObject implements ISimpleServer{

	private static final String CODE_ZIP = "server.zip";
	private static final String CODE_DIRECTORY = "build" + File.separator +
            "classes" + File.separator + "Client" + File.separator;
	
	private File binaryFile = null;
	private long binaryCheckSum = 0;
	
	public DownloadManager() throws RemoteException{
		super();
	}

	
	public synchronized void finished(){
	}
		
	private void createBinaryFile() throws IOException {
		Zipper zipper = new Zipper();
		
		File directory = new File(CODE_DIRECTORY);
		zipper.compress(directory, CODE_ZIP);
		binaryFile = new File(CODE_ZIP);
		binaryCheckSum = CheckSum.getCRC32(CODE_ZIP);
	}
	
	public IDownloadSession initializeBinaryDownload() throws IOException {
		if(binaryFile == null || !binaryFile.canRead()){
			synchronized (this) {
				//recheck again due to race conditions
				if(binaryFile == null || !binaryFile.canRead())
					createBinaryFile();
			}
		}
		
		try {
			return new DownloadSession(this,new DownloadSource(binaryFile.getName()),binaryCheckSum);
		} catch (RemoteException e) {e.printStackTrace();/*Not possible*/ } 
		catch (FileNotFoundException e) {e.printStackTrace();/*not possible*/ }
		
		return null;
	}

    public static void main(String[] args) {
        try {
            Registry rmi = LocateRegistry.createRegistry(PORT);
            ISimpleServer server = new DownloadManager();
            rmi.bind(LOOKUP_NAME, server);
        } catch (Exception ex) {
            System.out.println("Error");
            ex.printStackTrace();
        }
    }

}
