package Server;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.NoSuchElementException;

import Shared.DownloadBlock;

public class DownloadSource {

	private static final Integer DEFAULT_BLOCK_SIZE = 1024*1024;
	
	private FileInputStream stream;
	private DownloadBlock block;
	private int blockSize;
	
	public DownloadSource(String file) throws FileNotFoundException{
		this(file,DEFAULT_BLOCK_SIZE);
	}
	
	public DownloadSource(String file, int blockSize) throws FileNotFoundException {
		super();
		
		stream = new FileInputStream(file);
		this.blockSize = blockSize;
	}
	
	public DownloadBlock getCurrentBlock() throws IOException{
		if(block != null)
			return block;
		
		int result;
		byte[] block = new byte[blockSize];
		result = stream.read(block);
		//size down the block
		if(result < blockSize && result > -1){
			byte[] tmp = new byte[result];
			System.arraycopy(block, 0 , tmp,0, result);
			block = tmp;
		}
			
		//the last block will be an empty block
		if(result ==-1)
			block = new byte[0];
			
		return new DownloadBlock(result == -1, block);
	}
	
	public void nextBlock(){
		if(block != null && block.isLast())
			throw new NoSuchElementException();
		
		block = null;
	}
	
	public void finish() throws IOException{
		stream.close();
	}
}
